package jess;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * JessEvent listener broadcaster helper functions. 
 */

class JessEventSupport implements Serializable {
    
    private ArrayList m_listeners = new ArrayList();
    private JessEvent m_theEvent;
    private int m_eventMask = 0;

    JessEventSupport(Object source) {
        newEvent(source);
    }

    void newEvent(Object source) {
        m_theEvent = new JessEvent(source, 0, null);
    }

    public void addJessListener(JessListener jel) {
        m_listeners.add(jel);
    }

    public void removeJessListener(JessListener jel) {
        m_listeners.remove(jel);
    }


    public Iterator listJessListeners() {
        return m_listeners.iterator();
    }

    public int getEventMask() {
        return m_eventMask;
    }

    public void setEventMask(int i) {
        m_eventMask = i;
    }
    
    final void broadcastEvent(int type, Object data) throws JessException {

        // only broadcast active events
        if ((type & m_eventMask) == 0)
            return;

        // We lock this because it's cheaper than going in and out of
        // the vector methods over and over. We clone the vector so
        // that handlers added by an event handler won't be invoked
        // until the next time around.
        ArrayList snapshot;
        int size;

        synchronized (m_listeners) {
            if ((size = m_listeners.size()) == 0)
                return;
            
            snapshot = (ArrayList) m_listeners.clone();
        }

        synchronized (snapshot) {        
            for (int i=0; i<size; i++) {
                try {
                    m_theEvent.reset(type, data);
                    ((JessListener) snapshot.get(i)).
                        eventHappened(m_theEvent);
                } catch (JessException je) {
                    throw je;
                } catch (Exception e) {
                    throw new JessException("JessEventSupport.broadcastEvent",
                                            "Event handler threw an exception",
                                            e);                
                }
            }
        }
    }
}
