package jess;

import java.util.*;

/** **********************************************************************
 * A test that always passes, but makes calls with calltype LEFT instead of RIGHT.
 * @author Ernest J. Friedman-Hill
 ********************************************************************** */

class Node1RTL extends Node1 implements LogicalNode {    

    /**
     * Tokens that give logical support to facts
     */

    private HashMap m_logicalDepends;

    boolean callNodeRight(Token t) throws JessException {
        super.callNodeRight(t);
        if (m_logicalDepends != null)
            removeLogicalSupportFrom(t);
        
        passAlong(t);
        return true;
    }

    void passAlong(Token t) throws JessException {
        Node [] sa = m_succ;
        for (int j=0; j<m_nSucc; j++) 
            {
                Node s = sa[j];
                // System.out.println(this + " " +  t);
                s.callNodeLeft(t);
            }
    }

    public boolean equals(Object o) {
        return (o instanceof Node1RTL);
    }

    public String toString() { return "[Left input adapter]"; }

    public void dependsOn(Fact f, Token t) {
        if (m_logicalDepends == null)
            m_logicalDepends = new HashMap();

        ArrayList list = (ArrayList) m_logicalDepends.get(t);
        if (list == null) {
            list = new ArrayList();
            m_logicalDepends.put(t, list);
        }
        
        synchronized (list) {
            list.add(f);       
        }
    }

    // For testing only
    public Map getLogicalDependencies() {
        return m_logicalDepends;
    }

    private void removeLogicalSupportFrom(Token token) throws JessException {
        ArrayList list = (ArrayList) m_logicalDepends.get(token);
        if (list != null) 
            synchronized(list) {
                for (int i=0; i<list.size(); ++i) {
                    Fact f = (Fact) list.get(i);
                    Rete.getEngine().removeLogicalSupportFrom(token, f);
                }
            }
    }
}

